unit UClass.TAritmeticaBasica;

interface

uses
  System.Generics.Collections;

type
  // Type form integerlist
  TIntegerList = TList<Integer>;

  ///  <summary> Clase para clculos aritmticos </summary>
  TAritmeticaBasica = class
  strict private
    class var FInstance:TAritmeticaBasica;
  private
    function EsPrimo(x:integer):boolean;
    function MenorFactor(x:integer):integer;
  public
    ///  <summary> Devuelve si el nmero es primo </summary>
    function EsNumeroPrimo(const ANumero:int64):boolean;
    ///  <summary> Calcula el mximo comn Divisor de 2 nmeros enteros </summary>
    function MaximoComunDivisor(const ANumero1, ANumero2:int64):int64;
    ///  <summary> Calcula en Minimo comun Multiplo de 2 nmeros enteros </summary>
    function MinimoComunMultiplo(const ANumero1, ANumero2:int64):int64;
    ///  <summary> Calcula la factorizacion de un nmero entero </summary>
    procedure Factorizacion(ANumero: int64; ALista:TIntegerList);

    ///  <summary> Patron singleton para la clase </summary>
    class function GetInstance:TAritmeticaBasica;

    procedure GeneraEConvertError;
    procedure GeneraEArgumentException;

    function GeneraLeaksDeMemoria(ANumber:Integer):Integer;

  end;


implementation

uses
  System.Classes, System.SysUtils;


///  <summary> Devuelve el menos factor de division o el nmero si no lo tiene </summary>
function TAritmeticaBasica.MenorFactor(x: integer): integer;
var
  i: Integer;
begin
  Result := x;  // no tiene factor
  for i := 2 to x-1 do begin
    if ((x MOD i) = 0) then begin
      Result := i;
      Break;
    end;
  end;
end;


{https://www.saiyine.com/post.717-calcular-si-un-numero-es-primo-en-delphi---pascal.php}
function TAritmeticaBasica.EsPrimo(x:integer):boolean;
var
  i,r : longint;
begin
  r:=round(sqrt(x));
  for i:=2 to r do
    if (x mod i=0) then
      begin
        esPrimo:=false;
        exit;
      end;
  esPrimo:=true;
end;


function TAritmeticaBasica.EsNumeroPrimo(const ANumero: int64): boolean;
begin
  if (ANumero <= 0) then begin
    Result := False;
  end
  else begin
    Result := esPrimo(ANumero);
  end;
end;

procedure TAritmeticaBasica.Factorizacion(ANumero: int64; ALista:TIntegerList);
var
  x:integer;
begin
  if (ANumero <= 1) then begin
    Exit;
  end
  else if EsPrimo(ANumero) then begin
    ALista.Add(ANumero);
    Exit;
  end
  else begin
    x := MenorFactor(ANumero);
    if (x <> ANumero) then begin
      ALista.Add(x);
      ANumero := ANumero DIV x;
      // llamada recursiva
      Factorizacion(ANumero, ALista);
    end;
  end;
end;

procedure TAritmeticaBasica.GeneraEConvertError;
begin
  EConvertError.Create('Se genera error de la clase <EConvertError>');
end;

function TAritmeticaBasica.GeneraLeaksDeMemoria(ANumber:Integer):integer;
var
  SL:TStringList;
begin
  SL := TStringList.Create;
  SL.Add('Lunes');
  SL.Add('Martes');
  Result := ANumber;
end;

procedure TAritmeticaBasica.GeneraEArgumentException;
begin
  EArgumentException.Create('Se genera error de la clase <EArgumentException>');
end;

class function TAritmeticaBasica.GetInstance: TAritmeticaBasica;
begin
  if FInstance = nil then begin
    FInstance := TAritmeticaBasica.Create();
  end;
  Result := FInstance;
end;

function TAritmeticaBasica.MaximoComunDivisor(const ANumero1, ANumero2: int64): int64;
begin
  if ((ANumero1 mod ANumero2)= 0) then begin
    Result := ANumero2;
  end
  else begin
    Result := MaximoComunDivisor(ANumero2, ANumero1 mod ANumero2);
  end;
end;


function TAritmeticaBasica.MinimoComunMultiplo(const ANumero1, ANumero2: int64): int64;

begin
  Result := (ANumero1 * ANumero2) DIV MaximoComunDivisor(ANumero1, ANumero2);
end;

end.
